class Achievement
  attr_accessor :id
  attr_accessor :name
  attr_accessor :description
  attr_accessor :xp

  SPACE = " ".colorize(:yellow).colorize( :background => :black )
  MAX_LENGTH = 70
  PADDING = SPACE * 2
  PADDING_LENGTH = 2

  def initialize(id, name, description, xp, link, comment)
    @id = id
    @name = name
    @description = description
    @xp = xp
    @link = link
    @comment = comment
  end

  # public class methods
  class << self
    def locked_achievements(credentials)
      options = { headers: { "Content-Type" => "application/json" } }
      url = NagaApiClient.BASE_URL + "achievements?" + "authentication_token=" + credentials[:authentication_token] + "&email=" + credentials[:email]
      response = HTTParty.get(url, options)
      locked_achievements = {}
      response.parsed_response["locked_achievements"].each do |locked_achievement|
        achievement = Achievement.new(locked_achievement["id"], locked_achievement["name"], locked_achievement["description"], locked_achievement["xp"], locked_achievement["link"], locked_achievement["comment"])
        locked_achievements[achievement.name] = achievement
      end
      locked_achievements
    end
  end

  # public instance methods
  def unlock(credentials)
    url = NagaApiClient.BASE_URL + "achievements/unlock"
    options = { headers: { "Content-Type" => "application/x-www-form-urlencoded" } }
    options[:body] = "email=" + credentials[:email] + "&authentication_token=" + credentials[:authentication_token] + "&achievement_id=" + self.id.to_s
    response = HTTParty.post(url, options)
    parsed_response = response.parsed_response
    if parsed_response["error"]
      STDERR.puts(("Error: " + parsed_response["error"]).colorize(:red))
    else
      self.to_s_unlock
      if parsed_response["level_up"]
        puts "You just grew a level!".colorize(:black).colorize( :background => :green )
        puts "You are now Level #{parsed_response["level"]["number"].to_s} and have #{parsed_response["level"]["xp"].to_s} XP."
      end
    end
  end

  def to_s
    puts @id
    puts @name
    puts @description
    puts @xp
    puts @link
    puts @comment
  end

  def to_s_unlock
    puts SPACE * MAX_LENGTH
    puts PADDING + "Achievement unlocked!".ljust(MAX_LENGTH-PADDING_LENGTH).colorize(:yellow).colorize( :background => :black )
    puts PADDING + @name.ljust(MAX_LENGTH-PADDING_LENGTH).colorize(:yellow).colorize( :background => :black )
    description = "> " + @description
    puts PADDING + description.ljust(MAX_LENGTH-PADDING_LENGTH).colorize(:yellow).colorize( :background => :black )
    xp = "> " + @xp.to_s + " XP"
    if @comment && !@link
      puts PADDING + @comment.ljust(MAX_LENGTH-PADDING_LENGTH).colorize(:yellow).colorize( :background => :black )
    elsif @link && @comment
      puts PADDING + @link.ljust(MAX_LENGTH-PADDING_LENGTH).colorize(:yellow).colorize( :background => :black )
      puts PADDING + @comment.ljust(MAX_LENGTH-PADDING_LENGTH).colorize(:yellow).colorize( :background => :black )
    end
    puts PADDING + xp.ljust(MAX_LENGTH-PADDING_LENGTH).colorize(:yellow).colorize( :background => :black )
    puts SPACE * MAX_LENGTH
  end
end
